function [predictions errorrate confusionmat]= NNHausdorff(datacol,sIND,y,k,datacolN,sINDN,ynew)
% NNHausdorff : Prection of the class label based on the k-nearest neighbor
%  using Hausdorff distance.
%
% [predictions errorrate predictions]= NNHausdorff(datacol,sIND,y,k,datacolN,sINDN,ynew)
%   datacol, sIND, y : training dataset
%   datacolN, sINDN, ynew : testing dataset
%   k : number of neighbors.
%
% By Sungkyu Jung, June 2013.


if nargin < 6
    help NNHausdorff.m; return;
end

uniquesetindex=unique(sIND);
N = length(uniquesetindex);

[d Nn] = size(datacol);
label = unique(y);
K = length(label);

% processing the new data set
[d2 Nn2]=size(datacolN);
if d2 ~= d ;
    disp('Error from NNHausdorff: dimensions of input instances do not match');
    return;
end
uniquesetindextest=unique(sINDN);
Nnew = length(uniquesetindextest);


korig = k;

% Run k - Nearest Neighbor with Hausdorff distance
ind = zeros(Nnew,1); 
for i = 1:Nnew 
    datai = datacolN(:,sINDN == uniquesetindextest(i));
    % compute Hausdorff distance from each training set
    dist_tmp = zeros(N,1);
    for j = 1:N
        dist_tmp(j) = Hausdorff(datai, datacol(:,sIND == uniquesetindex(j)));
    end
    [sortedvalue orderindex]=sort(dist_tmp); 
    orderedy = y(orderindex);
    while(true)
        votes = zeros(K,1);
        for iclass = 1:K
            votes(iclass) = sum(orderedy(1:k) == label(iclass));
        end
        [tmp majorityvote]=max(votes);
        if k == length(orderedy);
            break;
        end
        if sum(votes >= tmp) > 1
            k = k + 1 ;
        else
            break;
        end
    end
    ind(i) = majorityvote;
    k = korig;
end

 
predictions = label(ind);

confusionmat = zeros(K);
for krow = 1:K
    for kcol = 1:K
        confusionmat(krow,kcol) = sum((ynew == label(krow)) & (predictions == label(kcol)));
    end
end
errorrate =  1 - (trace(confusionmat)/Nnew);


end







function hd = Hausdorff(A,B)
%
% computes the Hausdorff distance between two sets A, B by
% hd(A,B) = max p c A [ min q c B [ ||p-q|| ] ]
[a sizea]=size(A);
[a sizeb] =size(B);

maxA = 0;
% loop through all points in A looking for maxes
for p = 1:sizea
    % calculate the minimum distance from points in A to B
    minA = min(sum( bsxfun(@minus,A(:,p),B).^2, 1));
    if minA>maxA
        % new largest minimum for B
        maxA = minA;
    end
end
% repeat for points in A
maxB = 0;
for q = 1:sizeb
    minB = min(sum( bsxfun(@minus,B(:,1),A).^2, 1));
    if minB>maxB
        maxB = minB;
    end
end

hd = sqrt(max([maxA maxB]));
end






