function [predictions, errorrate, F, confusionmat]= SetFeatPredict(varargin)
% SETFEATPREDICT : Prection based on the rule trained from SetFeatlass.
%
% Pred = SetFeatPredict(X,delta,PCsMap) returns the prediction result
% of a new set X, using delta, PCsMap (from SetFeatlass);
%
% Pred = SetFeatPredict(X,v,delta,PCsMap) with new sets X, which is a
% collection of many sets indexed by the vector v.
%
% [Pred Errorrate ]= SetFeatPredict(X,v,delta,PCsMap,y) with new sets X,
% which is a collection of many sets indexed by the vector v. It gives
% prediction as well as the errorrate when the label 'y' is available.
%
% [Pred Errorrate F]= SetFeatPredict(X,v,delta,PCsMap) returns the new
%                   feature vector F (corresponding to X).
%
% [Pred Errorrate F confusionmat]= SetFeatPredict(X,v,delta,PCsMap,y,option)
% returns the confusion matrix if option = 1 (default = 0)
%
% By Sungkyu Jung, Feb 2013. June 2013.
%
% See also SetFeat, SetFeatMap, setFeatClass

if nargin < 3
    help SetFeatPredict.m; return;
end

if nargin == 3;
    X = varargin{1};
    delta = varargin{2};
    PCsMap = varargin{3};
    F = SetFeatMap(X,PCsMap);
    N = 1;
else
    X = varargin{1};
    sIND = varargin{2};
    delta = varargin{3};
    PCsMap = varargin{4};
    F = SetFeatMap(X,sIND,PCsMap);
    N = length(unique(sIND));
end

K = length(delta);
a = zeros(N,K);
for k = 1:K
    a(:,k) = delta{k}(F.mean,F.PCspace);
end
[tmp, ind] = max(a,[],2);
yulabel = PCsMap.label;
predictions = yulabel(ind);

if nargin > 4
    ynew = varargin{5}; 
    
    confusionmat = zeros(K);
    for krow = 1:K
        for kcol = 1:K
            confusionmat(krow,kcol) = sum((ynew == yulabel(krow)) & (predictions == yulabel(kcol)));
        end
    end    
    errorrate =  1 - (trace(confusionmat)/N);
    
    
    if nargin == 6
        printoption = varargin{6};
        if (printoption==1) && exist('printmat.m','file');
            rname = '';
            cname = '';
            for k = 1:K;
                rname = [rname 'true' num2str(k) ' '];
                cname = [cname 'pred' num2str(k) ' '];
            end
            
            printmat(confusionmat,'ConfusionMat',rname,cname);
        end
    end
    
    
end



