function [data y datacol sIND yIND] = simdata_models(imodel, params)
% SIMDATA_MODELS binary set classification simulations
% [data y datacol sIND yIND] = simdata_models(imodel, params);
% imodel = 1 : no covariance difference
% imodel = 2 : Wishart
% imodel = 3 : inverse Wishart
% imodel = 4 : von Mises Fisher
% params: (field) N1, N2,  p, (df, kappa, sigma, delta, s0, s1)
%
% fixed on 6/10. improved speed. inclusion of s0 and s1 for model 2
% fixed on 11/19.   AR(1)-like Toeplitz covariance matrix (Srivastava and
% Kubokawa (2007) included.
%
% Sungkyu Jung. Nov. 2013.

% retreive sampling plan
N1 = params.N1;
N2 = params.N2;
N = N1+N2;
p = params.p;
if isfield(params,'setsizeparams');
    setsizeparams = params.setsizeparams;
else
    setsizeparams = [20 5];
end
% minimum number of instances in each set
minn = 10;
vn = max(floor(randn(N,1)*setsizeparams(2)+setsizeparams(1)),minn); % vector of (n_1,...n_N)


% set default parameters for the model
df = min([10, p]);
kappa = 100;%*sqrt(p);
sigma = 3;
delta = 1;
s0 = ceil(p/10);
s1 = ceil(p/10);
useTAR = false;
c = 1;  % variance for set mean difference

% retreive parameters, if any
if isfield(params,'df');    df = params.df;end

if isfield(params,'kappa');    kappa = params.kappa;end

if isfield(params,'sigma');    sigma = params.sigma;end

if isfield(params,'delta');    delta = params.delta;end

if isfield(params,'s0');    s0 = params.s0;end

if isfield(params,'s1');    s1 = params.s1;end

if isfield(params,'useTAR'); useTAR = params.useTAR; end;

if isfield(params,'rho'); rho = params.rho; end;

if isfield(params,'c');    c = params.c;end


c = sqrt(c);


data = cell(1,N);
if ~useTAR
      switch imodel
        case 1    % mean difference / no cov difference.
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
              
            data = cell(1,N) ;
            for i = 1:N1
                data{i} = randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i = N1+1:N
                data{i} = randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
        case 2    % mean difference / cov difference - Wishart
            %mu1 = delta*[ones(s0,1); zeros(p-s0,1)]/sqrt(s0);
            %mu2 = -delta*[ones(s0,1); zeros(p-s0,1)]/sqrt(s0);
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            % V1 = eye(p) + sigma^2 / s1 * [ones(s1,1) ; zeros(p-s1,1)] * [ones(s1,1) ; zeros(p-s1,1)]';
            % V2 = eye(p) + sigma^2 / (p-s1) * [zeros(s1,1) ; ones(p-s1,1)] * [zeros(s1,1) ; ones(p-s1,1)]';
            
            V1 = eye(p);
            V1(1,1) = 1 + sigma.^2;
            V2 = eye(p);
            V2(2,2) = 1 + sigma.^2;
            
            for i = 1:N1
                [V,D] = eigs(wishrnd(V1,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i =  N1+1:N
                [V,D] = eigs(wishrnd(V2,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
        case 3    % mean difference / cov difference - inverse Wishart
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            V1 = eye(p);
            V1(1,1) = 1 + sigma.^2;
            V2 = eye(p);
            V2(2,2) = 1 + sigma.^2;
            
            df = p ; % deegree of freedom >= dimension
            
            data = cell(1,N) ;
            for i = 1:N1
                [V,D] = eigs(iwishrnd(V1,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i =  N1+1:N
                [V,D] = eigs(iwishrnd(V2,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
            
        case 4  % mean difference / cov difference - vMF
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            e1 = [1; zeros(p-1,1)];
            e2 = [0; 1; zeros(p-2,1)];
            for i = 1:N1
                dir = randvonMisesFisherm(p,1,kappa,e1);
                sqrtSigma = eye(p) + (sqrt(1+ sigma.^2)-1) *(dir*dir') ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i =  N1+1:N
                dir = randvonMisesFisherm(p,1,kappa,e2);
                sqrtSigma = eye(p) + (sqrt(1+ sigma.^2)-1) *(dir*dir') ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
            
        otherwise
            disp('Message from gendata_models.m: check your model number!')
            return;
    end  
    
    
    
else
    switch imodel
        case 1    % mean difference / no cov difference.
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            
            %mu1 = delta*[ones(s0,1); zeros(p-s0,1)]/sqrt(s0);
            %mu2 = zeros(p,1);
            SigmaTAR = TAR(rho,p);
            data = cell(1,N) ;
            for i = 1:N1
                data{i} = SigmaTAR*randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i = N1+1:N
                data{i} = SigmaTAR*randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
        case 2    % mean difference / cov difference - Wishart
            %mu1 = delta*[ones(s0,1); zeros(p-s0,1)]/sqrt(s0);
            %mu2 = -delta*[ones(s0,1); zeros(p-s0,1)]/sqrt(s0);
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            % V1 = eye(p) + sigma^2 / s1 * [ones(s1,1) ; zeros(p-s1,1)] * [ones(s1,1) ; zeros(p-s1,1)]';
            % V2 = eye(p) + sigma^2 / (p-s1) * [zeros(s1,1) ; ones(p-s1,1)] * [zeros(s1,1) ; ones(p-s1,1)]';
            
            SigmaTAR = TAR(rho,p);
            V1 = SigmaTAR;
            V1(1,1) = V1(1,1) + sigma.^2;
            V2 = SigmaTAR;
            V2(2,2) = V1(1,1) + sigma.^2;
            
            for i = 1:N1
                [V,D] = eigs(wishrnd(V1,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i =  N1+1:N
                [V,D] = eigs(wishrnd(V2,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
        case 3    % mean difference / cov difference - inverse Wishart
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            SigmaTAR = TAR(rho,p);
            V1 = SigmaTAR;
            V1(1,1) = V1(1,1) + sigma.^2;
            V2 = SigmaTAR;
            V2(2,2) = V1(1,1) + sigma.^2;
            
            df = p ; % deegree of freedom >= dimension
            
            data = cell(1,N) ;
            for i = 1:N1
                [V,D] = eigs(iwishrnd(V1,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i =  N1+1:N
                [V,D] = eigs(iwishrnd(V2,df)/df,df);
                sqrtSigma = V * D^(1/2) *  V' ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
            
        case 4  % mean difference / cov difference - vMF
            mu1 = [delta; zeros(p-1,1)];
            mu2 = zeros(p,1);
            SigmaTAR = TAR(rho,p);
            e1 = [1; zeros(p-1,1)];
            e2 = [0; 1; zeros(p-2,1)];
            for i = 1:N1
                dir = randvonMisesFisherm(p,1,kappa,e1);
                sqrtSigma = SigmaTAR + (sqrt(1+ sigma.^2)-1) *(dir*dir') ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu1 + c*randn(p,1),1,vn(i));
            end
            for i =  N1+1:N
                dir = randvonMisesFisherm(p,1,kappa,e2);
                sqrtSigma = SigmaTAR + (sqrt(1+ sigma.^2)-1) *(dir*dir') ;
                data{i} = sqrtSigma * randn(p,vn(i))+ repmat(mu2 + c*randn(p,1),1,vn(i));
            end
            
            
        otherwise
            disp('Message from gendata_models.m: check your model number!')
            return;
    end
end
y = [ones(N1,1);-ones(N-N1,1)] ;
setSizes = zeros(N,1);
datacol = [];
yIND = [];
sIND = [];
for i = 1:N;
    datacol = [datacol data{i}];
    setSizes(i) = size(data{i},2);
    yIND = [yIND y(i)*ones(1,setSizes(i))];
    sIND = [sIND i*ones(1,setSizes(i))];
end


end


function SigmaTAR = TAR(rho,p);
sigma = rand(p,1)*2/5+4/5;
covmat = zeros(p);
for i = 1:p;
    for j = 1:p;
        covmat(i,j) = rho.^(abs(i-j)^(1/7)) * sigma(i) * sigma(j) ;
    end
end
SigmaTAR = (covmat)^(1/2);
%
% for i = 1:p;
%     for j = 1:p;
%         covmat(i,j) = rho.^(abs(i-j)) * sigma(i) * sigma(j) ;
%     end
% end
% figure(1);clf; subplot(1,2,1);
% imagesc(SigmaTAR);subplot(1,2,2);
% imagesc(covmat);
end




